@@ -274,12 +274,12 @@ pub enum Instruction {
 ///     NumericInstruction::Convert(FloatType::F64, IntegerType::I32, SignExtension::Unsigned).into()
 /// );
 /// assert_eq!(
-///     Instruction::Numeric(NumericInstruction::ReinterpretFloat(IntegerType::I32, FloatType::F64)),
-///     NumericInstruction::ReinterpretFloat(IntegerType::I32, FloatType::F64).into()
+///     Instruction::Numeric(NumericInstruction::ReinterpretFloat(IntegerType::I32)),
+///     NumericInstruction::ReinterpretFloat(IntegerType::I32).into()
 /// );
 /// assert_eq!(
-///     Instruction::Numeric(NumericInstruction::ReinterpretInteger(FloatType::F64, IntegerType::I32)),
-///     NumericInstruction::ReinterpretInteger(FloatType::F64, IntegerType::I32).into()
+///     Instruction::Numeric(NumericInstruction::ReinterpretInteger(FloatType::F64)),
+///     NumericInstruction::ReinterpretInteger(FloatType::F64).into()
 /// );
 /// ```
 #[derive(Copy, Clone, Debug, PartialEq)]
@@ -387,9 +387,9 @@ pub enum NumericInstruction {
     /// fnn.convert_imm_sx
     Convert(FloatType, IntegerType, SignExtension),
     /// inn.reinterpret_fmm
-    ReinterpretFloat(IntegerType, FloatType),
+    ReinterpretFloat(IntegerType),
     /// fnn.reinterpret.imm
-    ReinterpretInteger(FloatType, IntegerType),
+    ReinterpretInteger(FloatType),
 }
 
 impl From<NumericInstruction> for Instruction {
@@ -685,36 +685,36 @@ impl From<TableInstruction> for Instruction {
 /// use wasm_ast::{MemoryInstruction, Instruction, NumberType, MemoryArgument, IntegerType, SignExtension};
 ///
 /// assert_eq!(
-///     Instruction::Memory(MemoryInstruction::Load(NumberType::I32, MemoryArgument::default())),
-///     MemoryInstruction::Load(NumberType::I32, MemoryArgument::default()).into()
+///     Instruction::Memory(MemoryInstruction::Load(NumberType::I32, MemoryArgument::default_offset(4))),
+///     MemoryInstruction::Load(NumberType::I32, MemoryArgument::default_offset(4)).into()
 /// );
 /// assert_eq!(
-///     Instruction::Memory(MemoryInstruction::Load8(IntegerType::I32, SignExtension::Signed, MemoryArgument::default())),
-///     MemoryInstruction::Load8(IntegerType::I32, SignExtension::Signed, MemoryArgument::default()).into()
+///     Instruction::Memory(MemoryInstruction::Load8(IntegerType::I32, SignExtension::Signed, MemoryArgument::default_offset(1))),
+///     MemoryInstruction::Load8(IntegerType::I32, SignExtension::Signed, MemoryArgument::default_offset(1)).into()
 /// );
 /// assert_eq!(
-///     Instruction::Memory(MemoryInstruction::Load16(IntegerType::I64, SignExtension::Unsigned, MemoryArgument::default())),
-///     MemoryInstruction::Load16(IntegerType::I64, SignExtension::Unsigned, MemoryArgument::default()).into()
+///     Instruction::Memory(MemoryInstruction::Load16(IntegerType::I64, SignExtension::Unsigned, MemoryArgument::default_offset(2))),
+///     MemoryInstruction::Load16(IntegerType::I64, SignExtension::Unsigned, MemoryArgument::default_offset(2)).into()
 /// );
 /// assert_eq!(
-///     Instruction::Memory(MemoryInstruction::Load32(SignExtension::Signed, MemoryArgument::default())),
-///     MemoryInstruction::Load32(SignExtension::Signed, MemoryArgument::default()).into()
+///     Instruction::Memory(MemoryInstruction::Load32(SignExtension::Signed, MemoryArgument::default_offset(4))),
+///     MemoryInstruction::Load32(SignExtension::Signed, MemoryArgument::default_offset(4)).into()
 /// );
 /// assert_eq!(
 ///     Instruction::Memory(MemoryInstruction::Store(NumberType::F64, MemoryArgument::default_offset(8))),
-///     MemoryInstruction::Store(NumberType::F64, MemoryArgument::new(Some(8), 0)).into()
+///     MemoryInstruction::Store(NumberType::F64, MemoryArgument::new(8, 0)).into()
 /// );
 /// assert_eq!(
-///     Instruction::Memory(MemoryInstruction::Store8(IntegerType::I32, MemoryArgument::default())),
-///     MemoryInstruction::Store8(IntegerType::I32, MemoryArgument::default()).into()
+///     Instruction::Memory(MemoryInstruction::Store8(IntegerType::I32, MemoryArgument::default_offset(1))),
+///     MemoryInstruction::Store8(IntegerType::I32, MemoryArgument::default_offset(1)).into()
 /// );
 /// assert_eq!(
-///     Instruction::Memory(MemoryInstruction::Store16(IntegerType::I64, MemoryArgument::default())),
-///     MemoryInstruction::Store16(IntegerType::I64, MemoryArgument::default()).into()
+///     Instruction::Memory(MemoryInstruction::Store16(IntegerType::I64, MemoryArgument::default_offset(2))),
+///     MemoryInstruction::Store16(IntegerType::I64, MemoryArgument::default_offset(2)).into()
 /// );
 /// assert_eq!(
-///     Instruction::Memory(MemoryInstruction::Store32(MemoryArgument::default())),
-///     MemoryInstruction::Store32(MemoryArgument::default()).into()
+///     Instruction::Memory(MemoryInstruction::Store32(MemoryArgument::default_offset(4))),
+///     MemoryInstruction::Store32(MemoryArgument::default_offset(4)).into()
 /// );
 /// assert_eq!(
 ///     Instruction::Memory(MemoryInstruction::Size),
@@ -942,20 +942,20 @@ pub enum BlockType {
 /// ```rust
 /// use wasm_ast::MemoryArgument;
 ///
-/// let argument = MemoryArgument::new(Some(4), 42);
+/// let argument = MemoryArgument::new(4, 42);
 ///
 /// assert_eq!(argument.offset(), 42);
-/// assert_eq!(argument.align(), Some(4));
+/// assert_eq!(argument.align(), 4);
 /// ```
 ///
 /// ## With Offset Only
 /// ```rust
 /// use wasm_ast::MemoryArgument;
 ///
-/// let argument = MemoryArgument::default_align(42);
+/// let argument = MemoryArgument::new(1, 42);
 ///
 /// assert_eq!(argument.offset(), 42);
-/// assert_eq!(argument.align(), None);
+/// assert_eq!(argument.align(), 1);
 /// ```
 ///
 /// ## With Alignment Only
@@ -965,52 +965,33 @@ pub enum BlockType {
 /// let argument = MemoryArgument::default_offset(4);
 ///
 /// assert_eq!(argument.offset(), 0);
-/// assert_eq!(argument.align(), Some(4));
+/// assert_eq!(argument.align(), 4);
 /// ```
 ///
 /// ## Default
 /// ```rust
 /// use wasm_ast::MemoryArgument;
 ///
-/// let argument = MemoryArgument::default();
+/// let argument = MemoryArgument::default_offset(1);
 ///
 /// assert_eq!(argument.offset(), 0);
-/// assert_eq!(argument.align(), None);
+/// assert_eq!(argument.align(), 1);
 /// ```
 #[derive(Copy, Clone, Debug, Eq, PartialEq)]
 pub struct MemoryArgument {
-    align: Option<u32>,
+    align: u32,
     offset: u32,
 }
 
 impl MemoryArgument {
     /// Creates a new memory argument with the given alignment and offset.
-    pub fn new(align: Option<u32>, offset: u32) -> Self {
+    pub fn new(align: u32, offset: u32) -> Self {
         MemoryArgument { align, offset }
     }
 
-    /// Creates a new memory argument with the default alignment and an offset of 0.
-    pub fn default() -> Self {
-        MemoryArgument {
-            offset: 0,
-            align: None,
-        }
-    }
-
-    /// Creates a new memory argument with the default alignment and the given offset.
-    pub fn default_align(offset: u32) -> Self {
-        MemoryArgument {
-            offset,
-            align: None,
-        }
-    }
-
     /// Creates a new memory argument with the default offset and the given alignment.
     pub fn default_offset(align: u32) -> Self {
-        MemoryArgument {
-            offset: 0,
-            align: Some(align),
-        }
+        MemoryArgument { offset: 0, align }
     }
 
     /// The static address offset of the memory instruction.
@@ -1019,9 +1000,7 @@ impl MemoryArgument {
     }
 
     /// The memory alignment of the instruction expressed as the exponent of a power of 2.
-    /// The default alignment cannot be computed without a number type.
-    /// Instead, returns None when the default alignment is set.
-    pub fn align(&self) -> Option<u32> {
+    pub fn align(&self) -> u32 {
         self.align
     }
 }
