@@ -355,7 +355,7 @@ pub fn parse_memory_instruction(input: &[u8]) -> IResult<&[u8], MemoryInstructio
 /// See <https://webassembly.github.io/spec/core/binary/instructions.html#memory-instructions>
 pub fn parse_memory_argument(input: &[u8]) -> IResult<&[u8], MemoryArgument> {
     map(tuple((parse_u32, parse_u32)), |(align, offset)| {
-        MemoryArgument::new(Some(align), offset)
+        MemoryArgument::new(align, offset)
     })(input)
 }
 
@@ -827,16 +827,16 @@ pub fn parse_numeric_instruction(input: &[u8]) -> IResult<&[u8], NumericInstruct
         )),
         alt((
             map(match_byte(0xBC), |_| {
-                NumericInstruction::ReinterpretFloat(IntegerType::I32, FloatType::F32)
+                NumericInstruction::ReinterpretFloat(IntegerType::I32)
             }),
             map(match_byte(0xBD), |_| {
-                NumericInstruction::ReinterpretFloat(IntegerType::I64, FloatType::F64)
+                NumericInstruction::ReinterpretFloat(IntegerType::I64)
             }),
             map(match_byte(0xBE), |_| {
-                NumericInstruction::ReinterpretInteger(FloatType::F32, IntegerType::I32)
+                NumericInstruction::ReinterpretInteger(FloatType::F32)
             }),
             map(match_byte(0xBF), |_| {
-                NumericInstruction::ReinterpretInteger(FloatType::F64, IntegerType::I64)
+                NumericInstruction::ReinterpretInteger(FloatType::F64)
             }),
         )),
         alt((
